#ifndef COMMON_H
#define COMMON_H

//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include <QCoreApplication>
#include <QStringList>
#include <QApplication>
#include <QMainWindow>
#include <QMenuBar>
#include <QDir>
#include <QFile>
#include <QAction>
#include <QDialog>
#include <QFileDialog>
#include <QErrorMessage>
#include <QThread>
#include <QProcess>
#include <QDateTime>
#include <QMessageBox>
#include <QSysInfo>
#include <QStackedWidget>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QBoxLayout>
#include <QPushButton>
#include <QSpinBox>
#include <QLineEdit>
#include <QCheckBox>
#include <QFormLayout>
#include <QLabel>
#include <QList>
#include <QTimerEvent>
#include <QShowEvent>
#include <QCursor>
#include <QUrl>
#include <QPixmap>
#include <QPainter>
#include <QScrollArea>
#include <QScrollBar>
#include <QTimer>
#include <QTime>
#include <QScreen>
#include <QToolButton>
#include <QPlainTextEdit>
#include <QCryptographicHash>
#include <QMutex>
#include <QMap>
#include <QThread>
#include <QSlider>
#include <QDebug>
#include <QGroupBox>
#include <QComboBox>
#include <QFileSystemWatcher>
#include <QRandomGenerator>
#include <QProcess>

#define APP_VERSION    "1.0.0"
#define APP_NAME       "JustMotion-Client"
#define APP_TARGET     "jmotion-client"
#define DEFAULT_PLAYER "vlc --no-audio --no-video-title-show --no-osd --repeat pls.m3u8"

#define PULL_RATE 700
#define THUMB_W   100
#define THUMB_H   75

class Actions;
class VidWidget;

enum ViewIndex
{
    VIDEO        = 0,
    NO_MEDIA     = 1,
    BUFFERING    = 2,
    END_OF_MEDIA = 3,
    PLAYBACK_ERR = 4
};

struct shared_t
{
    QString conf;
    QString buffPath;
    QString recPath;
    QString camName;
    QString playPath;
    QString recentsPath;
    QString appDataPath;
    QString appConfPath;
    QString hostAddr;
    QString hostKey;
    QString hostUser;
    QString hostPort;
    QString playerCmd;
    int     windowW;
    int     windowH;
    int     windowX;
    int     windowY;
    int     retCode;
};

struct ssh_opt
{
    QString pw;
    QString key;
    QString port;
    QString user;
};

QStringList lsFiles(const QString &path, const QStringList &filters);
QStringList lsFiles(const QString &path);
QStringList lsVidFiles(const QString &path);
QStringList lsImgFiles(const QString &path);
QStringList lsConfFiles(const QString &path);
QStringList lsDirsInDir(const QString &path);
QStringList parseArgs(const QByteArray &data, int maxArgs = -1, int *pos = nullptr);
QString     randPickItem(const QStringList &list);
bool        rdConf(const QString &filePath, shared_t *share);
bool        mkPath(const QString &path);
bool        delFolderContents(const QString &path);
bool        runCmd(const QStringList &args, QWidget *parent);
void        createOrUpdateRecent(shared_t *share);
void        wrConf(const QString &filePath, shared_t *share);
void        rdLine(const QString &param, const QString &line, QString *value);
void        rdLine(const QString &param, const QString &line, int *value);
void        rdLine(const QString &param, const QString &line, bool *value);
void        extCorrection(QString &ext);
void        resetSharedRes(shared_t *share);
void        buildGrid(QGridLayout *mainLayout, int parentWidth, const QList<QWidget*> &widList, QList<QWidget*> &spcList, int cellWidth, QWidget *parentWid);
void        elasticWid(QWidget *wid);
void        elasticWidLimitH(QWidget *wid);

class Proc : public QProcess
{
    Q_OBJECT

private:

    QStringList args;

private slots:

    void rdStdErr();
    void rdStdOut();

public:

    QString stdErr;
    QString stdOut;

    explicit Proc(const QString &cmd, QObject *parent = nullptr);

    int exec();
};

#endif // COMMON_H
