//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "host_dialog.h"

HostDialog::HostDialog(ssh_opt *opts, QWidget *parent) : QDialog(parent)
{
    sshOpts = opts;

    auto mainLayout = new QVBoxLayout(this);
    auto formWidget = new QWidget(this);
    auto formLayout = new QFormLayout(formWidget);

    hostPort = new QLineEdit(this);
    hostKey  = new QLineEdit(this);
    username = new QLineEdit(this);
    pw       = new QLineEdit(this);

    pw->setEchoMode(QLineEdit::PasswordEchoOnEdit);

    formLayout->addRow(new QLabel(tr("Host Port:"), this), hostPort);
    formLayout->addRow(new QLabel(tr("Host Key:"), this), hostKey);
    formLayout->addRow(new QLabel(tr("Username:"), this), username);
    formLayout->addRow(new QLabel(tr("Password:"), this), pw);

    hostPort->setText(sshOpts->port);
    hostKey->setText(sshOpts->key);
    username->setText(sshOpts->user);

    mainLayout->addWidget(formWidget, 0, Qt::AlignLeft);
    setContentsMargins(0,0,0,0);

    connect(hostPort, &QLineEdit::editingFinished, this, &HostDialog::wr);
    connect(hostKey,  &QLineEdit::editingFinished, this, &HostDialog::wr);
    connect(username, &QLineEdit::editingFinished, this, &HostDialog::wr);
    connect(pw,       &QLineEdit::editingFinished, this, &HostDialog::wr);
}

void HostDialog::closeEvent(QCloseEvent *event)
{
    wr();

    QDialog::closeEvent(event);
}

void HostDialog::wr()
{
    sshOpts->port = hostPort->text();
    sshOpts->key  = hostKey->text();
    sshOpts->user = username->text();
    sshOpts->pw   = pw->text();
}
