//    This file is part of JustMotion.

//    JustMotion is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    JustMotion is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "recents.h"

RecentItem::RecentItem(const QString &path, QComboBox *box, ssh_opt *opts, QObject *parent) : QObject(parent)
{
    shared_t conf;

    confPath = path;
    sshOpts  = opts;

    rdConf(path, &conf);

    setObjectName(conf.hostAddr);

    box->addItem(objectName());

    connect(box, &QComboBox::textActivated, this, &RecentItem::itemSelected);
}

void RecentItem::itemSelected(const QString &name)
{
    if (objectName() == name)
    {
        shared_t conf;

        rdConf(confPath, &conf);

        (*sshOpts).key  = conf.hostKey;
        (*sshOpts).port = conf.hostPort;
        (*sshOpts).user = conf.hostUser;
    }
}

Recents::Recents(shared_t *shareedRes, QComboBox *box, ssh_opt *opts, QObject *parent) : QObject(parent)
{
    share    = shareedRes;
    comboBox = box;
    sshOpts  = opts;

    mon.addPath(shareedRes->recentsPath);

    connect(&mon, &QFileSystemWatcher::directoryChanged, this, &Recents::scan);
    connect(this, &Recents::clear, box, &QComboBox::clear);

    scan();
}

void Recents::scan()
{
    emit clear();

    for (auto &&file: lsConfFiles(share->recentsPath))
    {
        auto item = new RecentItem(share->recentsPath + QDir::separator() + file, comboBox, sshOpts, this);

        connect(this, &Recents::clear, item, &RecentItem::deleteLater);
    }
}
